/* LIBDS
 * =====
 * This software is Copyright (c) 2002-03 Malcolm Smith.
 * No warranty is provided, including but not limited to
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * This code is licenced subject to the GNU General
 * Public Licence (GPL).  See the COPYING file for more.
 */

#ifndef WIN32_NO_CONFIG_H
#include "../config.h"
#else
#include "../winconf.h"
#endif
#include <DSComponent.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_IO_H
#include <io.h>
#endif

DSComponent::DSComponent(DSHashTable * hash)
{
	tshashreg=NULL;
	hashreg=hash;
	listreg=NULL;
}

DSComponent::DSComponent(DSList * list)
{
	tshashreg=NULL;
	hashreg=NULL;
	listreg=list;
}

DSComponent::DSComponent(DSTSHashTable * hash)
{
	tshashreg=hash;
	hashreg=NULL;
	listreg=NULL;
}

DSComponent::~DSComponent()
{
}

int DSComponent::loadcomponents(char * dir, char * key_symbol, char * value_symbol)
{
	int count;
#ifndef _WIN32_
	DIR * dh;
	struct dirent * thisfile;
	void * libhandle;
	int (*keysym)();
	void * valsym;
	char * tmp;
	int key;
	count=0;
	dh=opendir(dir);
	if (!dh) return count;
	while ((thisfile=readdir(dh))>0) {
		tmp=strrchr(thisfile->d_name,'.');
		if ((tmp)&&((strcmp(tmp,".so")==0)||(strcmp(tmp,".dylib")==0))) {
			tmp=(char *)malloc(thisfile->d_reclen+strlen(dir)+2);
			sprintf(tmp,"%s/%s",dir, thisfile->d_name);
			libhandle=dlopen(tmp, RTLD_NOW);
			free(tmp);
			if (libhandle) {
				keysym=(int (*)())dlsym(libhandle, key_symbol);
				tmp=dlerror();
				if (tmp==NULL) {
					valsym=dlsym(libhandle, value_symbol);
					if (dlerror()==NULL) {
						key=keysym();

						if (tshashreg) {
							tshashreg->lock();
							tshashreg->insert(key, valsym);
							tshashreg->unlock();
						}

						if (hashreg)
							hashreg->insert(key, valsym);
						if (listreg)
							listreg->append(key, valsym);
						count++;
					} else {
						dlclose(libhandle);
					}
				} else {
					dlclose(libhandle);
				}
				/* Note, not calling dlclose() */
			}
		}
		
	}
	return count;
#else
	HANDLE finder;
	WIN32_FIND_DATA wfd;
	HINSTANCE libhandle;
	BOOL finished;
	int (*keysym)();
	void * valsym;
	char * tmp;
	int key;
	count=0;
	tmp=(char *)malloc(strlen(dir)+7);
	sprintf(tmp, "%s\\*.DLL", dir);
	finder=FindFirstFile(tmp, &wfd);
	if (finder==INVALID_HANDLE_VALUE) return count;
	finished=FALSE;
	free(tmp);
	while (!finished) {
#ifdef HAVE_STRICMP
		if (stricmp(strrchr(wfd.cFileName,'.'),".DLL")==0) {
#else
#ifdef HAVE_STRCASECMP
		if (strcasecmp(strrchr(wfd.cFileName,'.'),".DLL")==0) {
#endif
#endif
			tmp=(char *)malloc(strlen(dir)+strlen(wfd.cFileName)+2);
			sprintf(tmp,"%s\\%s",dir, wfd.cFileName);
			libhandle=LoadLibrary(tmp);
			free(tmp);
			if (libhandle) {
				keysym=(int (*)())GetProcAddress(libhandle, key_symbol);
				if (keysym!=NULL) {
					valsym=(void *)GetProcAddress(libhandle, value_symbol);
					if (valsym!=NULL) {
						key=keysym();

						if (tshashreg) {
							tshashreg->lock();
							tshashreg->insert(key, valsym);
							tshashreg->unlock();
						}

						if (hashreg)
							hashreg->insert(key, valsym);
						if (listreg)
							listreg->append(key, valsym);
						count++;
						count++;
					} else {
						FreeLibrary(libhandle);
					}
				} else {
					FreeLibrary(libhandle);
				}
				/* Note, not calling FreeLibrary() */
			}
		}
		finished=!FindNextFile(finder, &wfd);
	}
	FindClose(finder);
	return count;
#endif
}


